package slike;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Orientation;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuBar;
import javafx.scene.control.MenuItem;
import javafx.scene.control.ToolBar;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.image.PixelReader;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.stage.FileChooser;
import javafx.stage.Screen;
import javafx.stage.Stage;

public class ImageApp extends Application {

	private FileChooser chooser = new FileChooser();
	private ImageView imageView = new ImageView();
	private Image original;

	@Override
	public void start(Stage stage) throws Exception {
		BorderPane root = new BorderPane();
		// izvlacimo dimenzije konkretnog ekrana
		double w = Screen.getPrimary().getBounds().getWidth();
		double h = Screen.getPrimary().getBounds().getHeight();
		// postavljamo dimenzije tako da zauzme ceo ekran
		Scene scene = new Scene(root, w, h - 100);

		MenuBar menuBar = new MenuBar();

		Menu fileMenu = new Menu("File");
		menuBar.getMenus().add(fileMenu);

		MenuItem miNew = new MenuItem("New");
		fileMenu.getItems().add(miNew);

		MenuItem miOpen = new MenuItem("Open");
		fileMenu.getItems().add(miOpen);
		miOpen.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent arg0) {
				// showOpenDialog je blokirajuca funkcija koja se
				// odblokira kada korisnik izadje iz open dijaloga i onda
				// vrati odabrani fajl
				File file = chooser.showOpenDialog(null);
				if (file != null) {
					loadImage(file);
				}
			}
		});

		MenuItem miSave = new MenuItem("Save");
		fileMenu.getItems().add(miSave);

		root.setTop(menuBar);

		// podesavanje regiona za sliku
		root.setCenter(imageView);

		// postavljanje trake sa alatima (toolbar)
		ToolBar toolBar = new ToolBar();
		toolBar.setOrientation(Orientation.VERTICAL);
		Button btnRevert = new Button("Revert");
		btnRevert.setOnAction(new EventHandler<ActionEvent>() {
			
			@Override
			public void handle(ActionEvent arg0) {
				imageView.setImage(original);
			}
		});
		
		Button btnRed = new Button("Make red");
		Button btnBlur = new Button("Blur");
		Button btnNeg = new Button("Negative");
		toolBar.getItems().add(btnRevert);
		toolBar.getItems().add(btnRed);
		toolBar.getItems().add(btnNeg);
		btnRed.setOnAction(new EventHandler<ActionEvent>() {

			@Override
			public void handle(ActionEvent arg0) {
				if (original != null) {
					Image image = getRedImage(original);
					imageView.setImage(image);
				}
			}
		});
		
		btnNeg.setOnAction(new EventHandler<ActionEvent>() {
			
			@Override
			public void handle(ActionEvent arg0) {
				if (original != null) {
					Image image = getNegativeImage(original);
					imageView.setImage(image);
				}
			}
		});

		toolBar.getItems().add(btnBlur);
		root.setRight(toolBar);

		stage.setScene(scene);
		stage.show();
	}

	private void loadImage(File file) {
		FileInputStream fis;
		try {
			fis = new FileInputStream(file);
			original = new Image(fis);
			imageView.setImage(original);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private Image getRedImage(Image img) {
		PixelReader pr = img.getPixelReader();
		int w = (int) img.getWidth();
		int h = (int) img.getHeight();
		WritableImage newImg = new WritableImage(w, h);
		PixelWriter pw = newImg.getPixelWriter();
		for (int i = 0; i < w; i++)
			for (int j = 0; j < h; j++) {
				Color c = pr.getColor(i, j);
				double redComp = c.getRed()+0.3;
				if(redComp>1)
					redComp=1;
				Color nc = new Color(redComp, c.getGreen(), c.getBlue(),c.getOpacity());
				pw.setColor(i, j, nc);
			}
		return newImg;
	}
	
	private Image getNegativeImage(Image img) {
		PixelReader pr = img.getPixelReader();
		int w = (int) img.getWidth();
		int h = (int) img.getHeight();
		WritableImage newImg = new WritableImage(w, h);
		PixelWriter pw = newImg.getPixelWriter();
		for (int i = 0; i < w; i++)
			for (int j = 0; j < h; j++) {
				Color c = pr.getColor(i, j);
				Color nc = new Color(1-c.getRed(), 1-c.getGreen(), 1-c.getBlue(),c.getOpacity());
				pw.setColor(i, j, nc);
			}
		return newImg;
	}


	public static void main(String[] args) {
		Application.launch(args);
	}

}
